<?php
namespace Estoque\Repositories;

class Inverotalote extends \Doctrine\ORM\EntityRepository
{


    CONST STATUS_N = "N";
    CONST P_STATUS_A = 'A';
    CONST P_STATUS_S = 'S';


    /**
     * Verifica se há inventário rotativo que usa lote pendente.
     *
     * @param $nrorg
     * @param $filial
     * @param $dtproc
     *
     * @return mixed
     */
    public function consultaInventPendenteRotaLote($nrorg, $filial, $dtproc){
        $sql = "SELECT I.*
                  FROM INVEROTALOTE I
                 WHERE I.NRORG      = :P_NRORG
                   AND I.CDFILIAL   = :P_CDFILIAL
                   AND I.DTINVELOTE = (SELECT MAX(DTINVELOTE)
                                       FROM INVEROTALOTE
                                      WHERE NRORG      = :P_NRORG
                                        AND CDFILIAL   = :P_CDFILIAL
                                        AND DTINVELOTE < :P_DTINVEROTA)
                   AND I.IDINVLOTEOK = :P_STATUS_N";

        $params = array(
            "P_CDFILIAL"   => $filial,
            "P_NRORG"      => $nrorg,
            "P_STATUS_N"   => self::STATUS_N,
            "P_DTINVEROTA" => $dtproc,
        );

        $entityManager = $this->getEntityManager();
        $resultSetMapping = new \Doctrine\ORM\Query\ResultSetMappingBuilder($entityManager);
        $resultSetMapping->addRootEntityFromClassMetadata('Estoque\Entities\Posestdia', "I");

        $result = $entityManager->createNativeQuery($sql, $resultSetMapping)->execute($params);
        return count($result)>0 ? current($result) : false;
    }


    /**
     * Insere Inventario rotativo de acordo com os produtos da filial e posição do estoque no dia.
     *
     * @param $nrorg
     * @param $filial
     * @param $dtInventario
     * @param $dataAtual
     * @param $operador
     *
     * @return \Doctrine\DBAL\Driver\Statement
     */
    public function insereInventarioRotativoLoteDtInventario($nrorg, $filial, $dtInventario, $dataAtual, $operador){
        $em = \Setforms2\Shared\Registry::getEntityManager();

        $insert = "INSERT INTO INVEROTALOTE
                    (CDFILIAL, CDPRODUTO, DTINVELOTE, QTESTOLOTE, QTINVELOTE, CDALMOXARIFE,
                     CDLOCALESTOQ, NRLOTEESTQ, NRSUBLOTE, NRORG, DTINCLUSAO,
                     NRORGINCLUSAO, CDOPERINCLUSAO)
                    (SELECT :P_FILIAL, D.CDPRODUTO, :P_DTINVEROTA, 0, 0, D.CDALMOXARIFE,
                            D.CDLOCALESTOQ, D.NRLOTEESTQ, D.NRSUBLOTE, D.NRORG, :P_DTINCLUSAO,
                            :P_NRORGINCLUSAO, :P_CDOPERINCLUSAO
                       FROM PRODUTO P, POSESTDIA D, PRODFILI F, PRODFILI C,
                            (SELECT CDFILIAL, CDALMOXARIFE, CDLOCALESTOQ, CDPRODUTO, NRLOTEESTQ,
                                    NRSUBLOTE, MAX(DTPOSIESTQ) AS DTPOSIESTQ, NRORG
                               FROM POSESTDIA
                              WHERE NRORG      = :P_NRORG
                                AND DTPOSIESTQ <= :P_DTINVEROTA
                              GROUP BY CDFILIAL, CDALMOXARIFE, CDLOCALESTOQ, CDPRODUTO, NRLOTEESTQ, NRSUBLOTE, DTPOSIESTQ, NRORG) AX
                      WHERE D.NRORG        = :P_NRORG
                        AND D.CDFILIAL     = :P_FILIAL
                        AND D.QTESTOQDIA   <> 0
                        AND D.CDPRODUTO    = P.CDPRODUTO
                        AND D.NRORG        = P.NRORG
                        AND P.IDPRODATIVPR = 'S'
                        AND P.CDPRODESTO   = F.CDPRODUTO
                        AND P.NRORG        = F.NRORG
                        AND F.IDCNTRESTOQ  = 'S'
                        AND F.IDPRODATIV   = 'A'
                        AND F.CDFILIAL     = D.CDFILIAL
                        AND F.NRORG        = D.NRORG
                        AND F.NRORG        = AX.NRORG
                        AND D.CDFILIAL     = AX.CDFILIAL
                        AND D.CDALMOXARIFE = AX.CDALMOXARIFE
                        AND D.CDLOCALESTOQ = AX.CDLOCALESTOQ
                        AND D.CDPRODUTO    = AX.CDPRODUTO
                        AND D.NRLOTEESTQ   = AX.NRLOTEESTQ
                        AND D.NRSUBLOTE    = AX.NRSUBLOTE
                        AND D.DTPOSIESTQ   = AX.DTPOSIESTQ
                        AND C.CDFILIAL     = D.CDFILIAL
                        AND C.NRORG        = D.NRORG
                        AND C.CDPRODUTO    = D.CDPRODUTO
                        AND C.IDPRODATIV   = 'A')";

        $params = array(
            "P_FILIAL" => $filial,
            "P_NRORG" => $nrorg,
            "P_DTINVEROTA" => $dtInventario,
            "P_DTINCLUSAO" => $dataAtual,
            "P_NRORGINCLUSAO" => $nrorg,
            "P_CDOPERINCLUSAO" => $operador,
        );

        return $em->getConnection()->executeQuery($insert, $params);
    }


    public function insereInventarioRotativoLoteLocal(
        $nrorg, $filial, $dtInventario, $almox, $local, $lote, $sublote,
        $produto, $qtdLote, $qtdEst, $identificador, $operador, $dataAtual
    ){
        $em = \Setforms2\Shared\Registry::getEntityManager();
        $insert = "INSERT INTO INVEROTALOTE
                          (NRORG, CDALMOXARIFE, CDLOCALESTOQ, NRLOTEESTQ, NRSUBLOTE, CDFILIAL,
                           DTINVELOTE, CDPRODUTO, QTINVELOTE, QTESTOLOTE, IDINVLOTEOK, NRORG,
                           CDOPERINCLUSAO, DTINCLUSAO)
                          (SELECT :P_NRORG, :P_CDALMOXARIFE, :P_CDLOCALESTOQ, :P_NRLOTEESTQ, :P_NRSUBLOTE, :P_CDFILIAL,
                                  :P_DTINVELOTE, :P_CDPRODUTO, :P_QTINVELOTE, :P_QTESTOLOTE, :P_IDINVLOTEOK, :P_NRORG,
                                  :P_CDOPERINCLUSAO, :P_DTINCLUSAO
                             FROM DUAL
                          MINUS
                            SELECT NRORG, CDALMOXARIFE, CDLOCALESTOQ, NRLOTEESTQ, NRSUBLOTE, CDFILIAL,
                                   DTINVELOTE, CDPRODUTO, :P_QTINVELOTE, :P_QTESTOLOTE, :P_IDINVLOTEOK, :P_NRORG,
                                  :P_CDOPERINCLUSAO, :P_DTINCLUSAO
                              FROM INVEROTALOTE
                             WHERE CDALMOXARIFE = :P_CDALMOXARIFE
                               AND CDLOCALESTOQ = CDLOCALESTOQ
                               AND NRLOTEESTQ   = :P_NRLOTEESTQ
                               AND NRSUBLOTE    = :P_NRSUBLOTE
                               AND CDFILIAL     = :P_CDFILIAL
                               AND NRORG        = :P_NRORG
                               AND DTINVELOTE   = :P_DTINVELOTE
                               AND CDPRODUTO    = :P_CDPRODUTO)";

        $params = array(
            "P_NRORG" => $nrorg,
            "P_CDFILIAL" => $filial,
            "P_CDALMOXARIFE" => $almox,
            "P_CDLOCALESTOQ" => $local,
            "P_NRLOTEESTQ" => $lote,
            "P_NRSUBLOTE" => $sublote,
            "P_DTINVELOTE" => $dtInventario,
            "P_CDPRODUTO" => $produto,
            "P_QTINVELOTE" => $qtdLote,
            "P_QTESTOLOTE" => $qtdEst,
            "P_IDINVLOTEOK" => $identificador,
            "P_CDOPERINCLUSAO" => $operador,
            "P_DTINCLUSAO" => $dataAtual,
        );

        return $em->getConnection()->executeQuery($insert, $params);
    }


    public function insereInventarioRotativoOK(
        $filial, $nrorg, $dtAtual, $operador, $dtPosicao, $dtPosAnt, $almox, $local
    ){
        $insert = "INSERT INTO INVEROTALOTE
                  (CDALMOXARIFE, CDLOCALESTOQ, NRLOTEESTQ, NRSUBLOTE, CDFILIAL,
                  DTINVELOTE, CDPRODUTO, QTINVELOTE, QTESTOLOTE, IDINVLOTEOK,
                  NRORG, DTINCLUSAO, CDOPERINCLUSAO,  NRORGINCLUSAO)
                 (SELECT D.CDALMOXARIFE, D.CDLOCALESTOQ, D.NRLOTEESTQ, D.NRSUBLOTE, D.CDFILIAL,
                        (:P_DTPOSICAOANT), D.CDPRODUTO,  0,  D.QTESTOQDIA, 'N'
                        D.NRORG, :P_DTINCLUSAO, :P_CDOPERINCLUSAO, :P_NRORGINCLUSAO
                    FROM PRODUTO P, POSESTDIA D, PRODFILI F, PRODFILI C,
                        (SELECT CDFILIAL, CDALMOXARIFE, CDLOCALESTOQ, CDPRODUTO,
                                NRLOTEESTQ, NRSUBLOTE, MAX(DTPOSIESTQ) AS DTPOSIESTQ
                           FROM POSESTDIA
                          WHERE NRORG = :P_NRORG
                            AND DTPOSIESTQ <= :P_DTPOSICAO
                          GROUP BY CDFILIAL, CDALMOXARIFE, CDLOCALESTOQ, CDPRODUTO, NRLOTEESTQ, NRSUBLOTE) A
                   WHERE D.CDFILIAL = :P_CDFILIAL
                     AND D.NRORG = :P_NRORG
                     AND D.CDALMOXARIFE = :P_CDALMOXARIFE
                     AND D.CDLOCALESTOQ = :P_CDLOCALESTOQ
                     AND D.CDPRODUTO = P.CDPRODUTO
                     AND D.NRORG = P.NRORG
                     AND P.CDPRODESTO = F.CDPRODUTO
                     AND P.IDPRODATIVPR = :P_STATUS_S
                     AND F.NRORG = D.NRORG
                     AND F.CDFILIAL = D.CDFILIAL
                     AND F.IDCNTRESTOQ = :P_STATUS_S
                     AND F.IDPRODATIV = :P_STATUS_A
                     AND D.CDFILIAL = A.CDFILIAL
                     AND D.CDALMOXARIFE = A.CDALMOXARIFE
                     AND D.CDLOCALESTOQ = A.CDLOCALESTOQ
                     AND D.CDPRODUTO = A.CDPRODUTO
                     AND D.NRLOTEESTQ = A.NRLOTEESTQ
                     AND D.NRSUBLOTE = A.NRSUBLOTE
                     AND D.DTPOSIESTQ = A.DTPOSIESTQ
                     AND C.NRORG = D.NRORG
                     AND C.CDFILIAL = D.CDFILIAL
                     AND C.CDPRODUTO = D.CDPRODUTO
                     AND C.IDPRODATIV = :P_STATUS_A)";

        $params = array(
            "P_FILIAL" => $filial,
            "P_NRORG" => $nrorg,
            "P_DTINCLUSAO" => $dtAtual,
            "P_CDOPERINCLUSAO" => $operador,
            "P_NRORGINCLUSAO" => $nrorg,
            "P_DTPOSICAO" => $dtPosicao,
            "P_DTPOSICAOANT" => $dtPosAnt,
            "P_CDALMOXARIFE" => $almox,
            "P_CDLOCALESTOQ" => $local,
            "P_STATUS_S" => self::P_STATUS_S,
            "P_STATUS_A" => self::P_STATUS_A,
        );

        $em = \Setforms2\Shared\Registry::getEntityManager();
        return $em->getConnection()->executeQuery($insert, $params);
    }



    /**
     * @param $nrorg
     * @param $filial
     * @param $dtInventario
     * @param $almox
     * @param $local
     * @param $dataAtual
     * @param $operador
     *
     * @return \Doctrine\DBAL\Driver\Statement
     */
    public function insereInventarioRotativoLotePosicao(
        $nrorg, $filial, $dtInventario, $almox, $local, $dataAtual, $operador
    ){
        $em = \Setforms2\Shared\Registry::getEntityManager();

        $insert = "INSERT INTO INVEROTALOTE
                  (CDFILIAL, CDPRODUTO, DTINVELOTE, QTESTOLOTE, QTINVELOTE, CDALMOXARIFE,
                   CDLOCALESTOQ, NRLOTEESTQ, NRSUBLOTE, NRORG, DTINCLUSAO,
                   NRORGINCLUSAO, CDOPERINCLUSAO)
                  (SELECT :P_FILIAL, C.CDPRODUTO, :P_DTINVEROTA, 0, 0, :P_CDALMOXARIFE,
                          :P_CDLOCALESTOQ, P.NRLOTEESTQ, P.NRSUBLOTE, P.NRORG, :P_DTINCLUSAO,
                          :P_NRORGINCLUSAO, :P_CDOPERINCLUSAO
                     FROM CURVAABC C, POSESTDIA P
                    WHERE C.NRORG        = :P_NRORG
                      AND C.CDFILIAL     = :P_FILIAL
                      AND C.CDALMOXARIFE = :P_CDALMOXARIFE
                      AND C.CDLOCALESTOQ = :P_CDLOCALESTOQ
                      AND C.NRORG        = P.NRORG
                      AND C.CDFILIAL     = P.CDFILIAL
                      AND C.CDALMOXARIFE = P.CDALMOXARIFE
                      AND C.CDLOCALESTOQ = P.CDLOCALESTOQ
                      AND C.CDPRODUTO    = P.CDPRODUTO
                      AND P.QTESTOQDIA   <> 0
                      AND P.DTPOSIESTQ   = (SELECT MAX(DTPOSIESTQ)
                                              FROM POSESTDIA
                                             WHERE NRORG        = :P_NRORG
                                               AND CDFILIAL     = :P_FILIAL
                                               AND CDALMOXARIFE = :P_CDALMOXARIFE
                                               AND CDLOCALESTOQ = :P_CDLOCALESTOQ
                                               AND NRLOTEESTQ   = P.NRLOTEESTQ
                                               AND CDPRODUTO    = P.CDPRODUTO
                                               AND NRSUBLOTE    = P.NRSUBLOTE
                                               AND DTPOSIESTQ   <= :P_DTINVEROTA))";

        $params = array(
            "P_FILIAL" => $filial,
            "P_NRORG" => $nrorg,
            "P_DTINVEROTA" => $dtInventario,
            "P_CDALMOXARIFE" => $almox,
            "P_CDLOCALESTOQ" => $local,
            "P_DTINCLUSAO" => $dataAtual,
            "P_NRORGINCLUSAO" => $nrorg,
            "P_CDOPERINCLUSAO" => $operador,
        );

        return $em->getConnection()->executeQuery($insert, $params);
    }

    public function inventarioLoteOk($cdfilial, $cdalmoxarife, $cdlocalestoq, $dtinvelote) {

        $sql =
            "SELECT IDINVLOTEOK
               FROM INVEROTALOTE
              WHERE CDFILIAL = :P_CDFILIAL
                AND CDALMOXARIFE = :P_CDALMOXARIFE
                AND CDLOCALESTOQ = :P_CDLOCALESTOQ
                AND IDINVLOTEOK = 'N'
                AND DTINVELOTE = (SELECT MAX(DTINVELOTE)
                                    FROM INVEROTA
                                   WHERE CDFILIAL = :P_CDFILIAL
                                     AND CDALMOXARIFE = :P_CDALMOXARIFE
                                     AND CDLOCALESTOQ = :P_CDLOCALESTOQ
                                     AND DTINVELOTE < :P_DTINVELOTE
                                )";

        $params = array(
            'P_CDFILIAL'     => $cdfilial,
            'P_CDALMOXARIFE' => $cdalmoxarife,
            'P_CDLOCALESTOQ' => $cdlocalestoq,
            'P_DTINVELOTE'   => $dtinvelote
        );

        return $this->getEntityManager()->getConnection()->fetchAll($sql, $params);
    }

}